{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import libraries"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import ee\n",
    "import geemap"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Create an interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map = geemap.Map(center=[40, -100], zoom=4)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Add Earth Engine Python script"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Add Earth Engine dataset\n",
    "image = ee.Image(\"USGS/SRTMGL1_003\")\n",
    "\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "#  Chapter:      A2.5 Water Balance and Drought\n",
    "#  Checkpoint:   A25d\n",
    "#  Authors:      Ate Poortinga, Quyen Nguyen, Nyein Soe Thwal, Andréa Puzzi Nicolau\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "\n",
    "# Import the Lower Mekong boundary.\n",
    "mekongBasin = ee.FeatureCollection(\n",
    "    'projects/gee-book/assets/A2-5/lowerMekongBasin')\n",
    "\n",
    "# Center the map.\n",
    "Map.centerObject(mekongBasin, 5)\n",
    "\n",
    "# Add the Lower Mekong Basin boundary to the map.\n",
    "Map.addLayer(mekongBasin, {}, 'Lower Mekong basin')\n",
    "\n",
    "# Set start and end years.\n",
    "startYear = 2010\n",
    "endYear = 2020\n",
    "\n",
    "# Create two date objects for start and end years.\n",
    "startDate = ee.Date.fromYMD(startYear, 1, 1)\n",
    "endDate = ee.Date.fromYMD(endYear + 1, 1, 1)\n",
    "\n",
    "# Make a list with years.\n",
    "years = ee.List.sequence(startYear, endYear)\n",
    "\n",
    "# Make a list with months.\n",
    "months = ee.List.sequence(1, 12)\n",
    "\n",
    "# Import the CHIRPS dataset.\n",
    "CHIRPS = ee.ImageCollection('UCSB-CHG/CHIRPS/PENTAD')\n",
    "\n",
    "# Filter for relevant time period.\n",
    "CHIRPS = CHIRPS.filterDate(startDate, endDate)\n",
    "\n",
    "# Import the MOD16 dataset.\n",
    "mod16 = ee.ImageCollection('MODIS/006/MOD16A2').select('ET')\n",
    "\n",
    "# Filter for relevant time period.\n",
    "mod16 = mod16.filterDate(startDate, endDate)\n",
    "\n",
    "# Import and filter the MOD13 dataset.\n",
    "mod13 = ee.ImageCollection('MODIS/006/MOD13A1')\n",
    "mod13 = mod13.filterDate(startDate, endDate)\n",
    "\n",
    "# Select the EVI.\n",
    "EVI = mod13.select('EVI')\n",
    "\n",
    "# Import and filter the MODIS Terra surface reflectance dataset.\n",
    "mod09 = ee.ImageCollection('MODIS/006/MOD09A1')\n",
    "mod09 = mod09.filterDate(startDate, endDate)\n",
    "\n",
    "# We use a function to remove clouds and cloud shadows.\n",
    "# We map over the mod09 image collection and select the StateQA band.\n",
    "# We mask pixels and return the image with clouds and cloud shadows masked.\n",
    "\n",
    "def func_rzy(image):\n",
    "    quality = image.select('StateQA')\n",
    "    mask = image.And(quality.bitwiseAnd(1).eq(\n",
    "            0)) # No clouds. \\\n",
    "        .And(quality.bitwiseAnd(2).eq(0)); \n",
    "\n",
    "    return image.updateMask(mask)\n",
    "\n",
    "mod09 = mod09.map(func_rzy)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# We use a function to calculate the Moisture Stress Index.\n",
    "# We map over the mod09 image collection and select the NIR and SWIR bands\n",
    "# We set the timestamp and return the MSI.\n",
    "\n",
    "def func_fjo(image):\n",
    "    nirband = image.select('sur_refl_b02')\n",
    "    swirband = image.select('sur_refl_b06')\n",
    "\n",
    "    msi = swirband.divide(nirband).rename('MSI') \\\n",
    "        .set('system:time_start', image.get(\n",
    "            'system:time_start'))\n",
    "    return msi\n",
    "\n",
    "MSI = mod09.map(func_fjo)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# We apply a nested loop where we first iterate over\n",
    "# the relevant years and then iterate over the relevant\n",
    "# months. The function returns an image with bands for\n",
    "# water balance (wb), rainfall (P), evapotranspiration (ET),\n",
    "# EVI and MSI for each month. A flatten is applied to\n",
    "# convert an collection of collections\n",
    "# into a single collection.\n",
    "ic = ee.ImageCollection.fromImages(\n",
    "\n",
    "def func_gjw(y):\n",
    "        return months.map(function(m) {\n",
    "            # Calculate rainfall.\n",
    "            P = CHIRPS.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .sum()\n",
    "\n",
    "            # Calculate evapotranspiration.\n",
    "            ET = mod16.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .sum() \\\n",
    "                .multiply(0.1)\n",
    "\n",
    "            # Calculate EVI.\n",
    "            evi = EVI.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .mean() \\\n",
    "                .multiply(0.0001)\n",
    "\n",
    "            # Calculate MSI.\n",
    "            msi = MSI.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .mean()\n",
    "\n",
    "            # Calculate monthly water balance.\n",
    "            wb = P.subtract(ET).rename('wb')\n",
    "\n",
    "            # Return an image with all images as bands.\n",
    "            return ee.Image.cat([wb, P, ET, evi, msi]) \\\n",
    "                .set('year', y) \\\n",
    "                .set('month', m) \\\n",
    "                .set('system:time_start', ee.Date \\\n",
    "                    .fromYMD(y, m, 1))\n",
    "\n",
    "        })\n",
    "\n",
    "    years.map(func_gjw\n",
    ").flatten()\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    ").flatten()\n",
    ")\n",
    "\n",
    "# Add the mean monthly EVI and MSI to the map.\n",
    "eviVis = {\n",
    "    'min': 0,\n",
    "    'max': 0.7,\n",
    "    'palette': 'red, orange, yellow, green, darkgreen'\n",
    "}\n",
    "\n",
    "Map.addLayer(ic.select('EVI').mean().clip(mekongBasin),\n",
    "    eviVis,\n",
    "    'EVI')\n",
    "\n",
    "msiVis = {\n",
    "    'min': 0.25,\n",
    "    'max': 1,\n",
    "    'palette': 'darkblue, blue, yellow, orange, red'\n",
    "}\n",
    "\n",
    "Map.addLayer(ic.select('MSI').mean().clip(mekongBasin),\n",
    "    msiVis,\n",
    "    'MSI')\n",
    "\n",
    "# Define the water balance chart and print it to the console.\n",
    "chartWB =\n",
    "    ui.Chart.image.series({\n",
    "        'imageCollection': ic.select(['wb', 'precipitation', 'ET']),\n",
    "        'region': mekongBasin,\n",
    "        'reducer': ee.Reducer.mean(),\n",
    "        'scale': 5000,\n",
    "        'xProperty': 'system:time_start'\n",
    "    }) \\\n",
    "    .setSeriesNames(['wb', 'P', 'ET']) \\\n",
    "    .setOptions({\n",
    "        'title': 'water balance',\n",
    "        'hAxis': {\n",
    "            'title': 'Date',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'vAxis': {\n",
    "            'title': 'Water (mm)',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'lineWidth': 1,\n",
    "        'colors': ['green', 'blue', 'red'],\n",
    "        'curveType': 'function'\n",
    "    })\n",
    "\n",
    "# Print the water balance chart.\n",
    "print(chartWB)\n",
    "\n",
    "# Define the indices chart and print it to the console.\n",
    "chartIndices =\n",
    "    ui.Chart.image.series({\n",
    "        'imageCollection': ic.select(['EVI', 'MSI']),\n",
    "        'region': mekongBasin,\n",
    "        'reducer': ee.Reducer.mean(),\n",
    "        'scale': 5000,\n",
    "        'xProperty': 'system:time_start'\n",
    "    }) \\\n",
    "    .setSeriesNames(['EVI', 'MSI']) \\\n",
    "    .setOptions({\n",
    "        'title': 'Monthly indices',\n",
    "        'hAxis': {\n",
    "            'title': 'Date',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'vAxis': {\n",
    "            'title': 'Index',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'lineWidth': 1,\n",
    "        'colors': ['darkgreen', 'brown'],\n",
    "        'curveType': 'function'\n",
    "    })\n",
    "\n",
    "# Print the indices chart.\n",
    "print(chartIndices)\n",
    "\n",
    "# -----------------------------------------------------------------------------------\n",
    "# CHECKPOINT\n",
    "# -----------------------------------------------------------------------------------"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Display the interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map"
   ]
  }
 ],
 "metadata": {
  "anaconda-cloud": {},
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
